@@ -66,10 +66,6 @@
 #include <sys/resource.h>
 #include <sys/time.h>
 
-#if defined(IRIX)
- #define PATH_MAX FILENAME_MAX
-#endif
-
 #ifndef USE_USLEEP
  #include <time.h>
 #endif
@@ -192,8 +188,6 @@ void changePidFileGroup(const TCHAR* filename, gid_t newGroup) {
  * @param pid      pid to write in the file.
  * @param newUmask Umask to use when creating the file.
  * @param newGroup Group to use when creating the file.
- * @param strict   If true then an error will be reported and the call will fail if the
- *                 file already exists.
  *
  * @return 1 if there was an error, 0 if Ok.
  */
