@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2018 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2019 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -68,6 +68,7 @@ public abstract class AbstractActionApp
     
     private long m_eventMask = 0xffffffffffffffffL;
     private int m_slowSeconds = 0;
+    private int m_suspendSeconds = 0;
     private String m_serviceName = "testWrapper";
     private String m_consoleTitle = "Java Service Wrapper";
     private String m_childCommand = "ls";
@@ -174,6 +175,11 @@ public abstract class AbstractActionApp
         m_slowSeconds = slowSeconds;
     }
     
+    protected void setSuspendSeconds( int suspendSeconds )
+    {
+        m_suspendSeconds = suspendSeconds;
+    }
+    
     protected void setServiceName( String serviceName )
     {
         m_serviceName = serviceName;
@@ -296,6 +302,16 @@ public abstract class AbstractActionApp
         {
             WrapperManager.accessViolationNative();
             
+        }
+        else if ( action.equals( "exception_native" ) )
+        {
+            WrapperManager.raiseExceptionNative( 0x12345678 );
+            
+        }
+        else if ( action.equals( "ff_exception_native" ) )
+        {
+            WrapperManager.raiseFailFastExceptionNative();
+            
         }
         else if ( action.equals( "appear_hung" ) )
         {
@@ -354,6 +370,50 @@ public abstract class AbstractActionApp
             m_err.setDeadlock( true );
             
         }
+        else if ( action.equals( "suspend_timeouts" ) )
+        {
+            if ( WrapperManager.isStandardEdition() )
+            {
+                WrapperManager.suspendTimeouts( m_suspendSeconds );
+            }
+            else
+            {
+                System.out.println( Main.getRes().getString( "Suspending/Resuming timeouts requires the Standard Edition.") );
+            }
+        }
+        else if ( action.equals( "suspend_timeouts_10" ) )
+        {
+            if ( WrapperManager.isStandardEdition() )
+            {
+                WrapperManager.suspendTimeouts( 10 );
+            }
+            else
+            {
+                System.out.println( Main.getRes().getString( "Suspending/Resuming timeouts requires the Standard Edition.") );
+            }
+        }
+        else if ( action.equals( "suspend_timeouts_60" ) )
+        {
+            if ( WrapperManager.isStandardEdition() )
+            {
+                WrapperManager.suspendTimeouts( 60 );
+            }
+            else
+            {
+                System.out.println( Main.getRes().getString( "Suspending/Resuming timeouts requires the Standard Edition.") );
+            }
+        }
+        else if ( action.equals( "resume_timeouts" ) )
+        {
+            if ( WrapperManager.isStandardEdition() )
+            {
+                WrapperManager.resumeTimeouts();
+            }
+            else
+            {
+                System.out.println( Main.getRes().getString( "Suspending/Resuming timeouts requires the Standard Edition.") );
+            }
+        }
         else if ( action.equals( "users" ) )
         {
             if ( !m_users )
@@ -404,7 +464,6 @@ public abstract class AbstractActionApp
                 {
                     public void run()
                     {
-                        System.out.println();
                         System.out.println( Main.getRes().getString( "Start prompting for actions." ) );
                         try
                         {
@@ -414,7 +473,25 @@ public abstract class AbstractActionApp
                             {
                                 do {
                                     System.out.println( Main.getRes().getString( "Input an action (''help'' for a list of actions):") );
-                                    line = r.readLine();
+                                    
+                                    int count = 0;
+                                    int maxTries = 10;
+                                    while( true ) {
+                                        /* On some old JVMs, we may get an "Interrupted system call" exception if a child process is writing some output while tying to read from here.
+                                         *  It would most likely work if we try again though. */
+                                        try
+                                        {
+                                            line = r.readLine();
+                                            break;
+                                        }
+                                        catch ( IOException e )
+                                        {
+                                            if ( ( e.toString().indexOf( "Interrupted system call" ) == -1 ) || ( ++count == maxTries ) )
+                                            {
+                                                throw e;
+                                            }
+                                        }
+                                    }
                                     if ((line != null) && (!line.equals(""))) {
                                         System.out.println(Main.getRes().getString( "Read action: {0}", line ) );
                                         if ( !doAction( line ) )
@@ -759,6 +836,11 @@ public abstract class AbstractActionApp
         System.err.println( Main.getRes().getString( "   stopimmediate1           : Calls WrapperManager.stopImmediate(1)" ) );
         System.err.println( Main.getRes().getString( "  Actions which should cause the Wrapper to restart the JVM:" ) );
         System.err.println( Main.getRes().getString( "   access_violation_native  : Calls WrapperManager.accessViolationNative()" ) );
+        if ( WrapperManager.isWindows() )
+        {
+            System.err.println( Main.getRes().getString( "   exception_native         : Calls WrapperManager.raiseExceptionNative()" ) );
+            System.err.println( Main.getRes().getString( "   ff_exception_native      : Calls WrapperManager.raiseFailFastExceptionNative()" ) );
+        }
         System.err.println( Main.getRes().getString( "   appear_hung              : Calls WrapperManager.appearHung()" ) );
         System.err.println( Main.getRes().getString( "   halt0                    : Calls Runtime.getRuntime().halt(0)" ) );
         System.err.println( Main.getRes().getString( "   halt1                    : Calls Runtime.getRuntime().halt(1)" ) );
@@ -773,6 +855,9 @@ public abstract class AbstractActionApp
         System.err.println( Main.getRes().getString( "   ignore_events            : Makes this application ignore control events." ) );
         System.err.println( Main.getRes().getString( "   dump                     : Calls WrapperManager.requestThreadDump()" ) );
         System.err.println( Main.getRes().getString( "   deadlock_out             : Deadlocks the JVM''s System.out and err streams." ) );
+        System.err.println( Main.getRes().getString( "   suspend_timeouts_10      : Calls WrapperManager.suspendTimeouts(10)  (Standard, Professional)" ) );
+        System.err.println( Main.getRes().getString( "   suspend_timeouts_60      : Calls WrapperManager.suspendTimeouts(60)  (Standard, Professional)" ) );
+        System.err.println( Main.getRes().getString( "   resume_timeouts          : Calls WrapperManager.resumeTimeouts()  (Standard, Professional)" ) );
         System.err.println( Main.getRes().getString( "   users                    : Start polling the current and interactive users." ) );
         System.err.println( Main.getRes().getString( "   groups                   : Start polling the current and interactive users with groups." ) );
         System.err.println( Main.getRes().getString( "   console                  : Prompt for actions in the console." ) );
